#! /usr/bin/env python
# -*- coding: UTF-8 -*- 

import os
import re
import gettext
import dbus
import dbus.mainloop.glib
from gi.repository import GObject

APP_NAME = "fingerprint-pam"
LOCALE_DIR = os.path.abspath("/usr/share/locale/")
gettext.bindtextdomain(APP_NAME, LOCALE_DIR)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

class FPData(object):
    '''fingerprint data class
    '''
    
    def __init__(self, username, fpId, alias):
        self.username = username
        self.fpId = int(fpId)
        self.alias = alias
        pass
    
    def toString(self):
        return self.username + ':' + str(self.fpId) + ':' + self.alias




class FPConfig(object):
    '''read and write /etc/fingerprint
    '''

    def __init__(self):
        self.configPath = '/etc/fingerprint'
        self.config = None
        self.fpData = []

        self.parseConfig()
        pass

    def openConfigFile(self):
        self.config = open(self.configPath, 'r')
        pass

    def parseConfig(self):
        if self.config is None:
            self.openConfigFile()

        if self.fpData and isinstance(self.fpData, list) and len(self.fpData) > 0:
            return

        line = self.config.readline()

        self.fpData = []
        while line:
            line = line.strip()
            fpData = line.split(':')
            if fpData and isinstance(fpData, list) and len(fpData) == 3:
                fpData = FPData(fpData[0], fpData[1], fpData[2])
                self.fpData.append(fpData)
            line = self.config.readline()
            pass
        pass

    def commitConfig(self):
        # self.config.seek(0)
        # self.config.truncate()
        # self.config.seek(0)

        confStr = ''
        try:
            for fpData in self.fpData:
                tmpStr = fpData.toString() + '\n'
                # self.config.write(tmpStr)
                confStr = confStr + tmpStr
            # self.config.flush()

            
        except Exception, error:
            return ''

        return confStr

    def margeConfig(self, userList):
        if len(userList) < 1:
            return
        username = userList[0].username

        self.fpData = filter(lambda x: x.username != username, self.fpData)

        for fpData in userList:
            self.fpData.append(fpData)

        pass

    def getUserFPData(self, username):
        self.parseConfig()
        userFPData = []

        for fpData in self.fpData:
            if fpData.username == username:
                userFPData.append(fpData)

        return userFPData

    def get_current_user_info_list(self):
        username = os.getlogin()
        return self.getUserFPData(username)
    
    def insertFPData(self, fpData):
        if fpData is None or not isinstance(fpData, FPData):
            return _('not a fingerprint data!')
        
        if self.findConfigUseFPId(fpData.fpId):
            return _('fingerprint data already exists!')
            
        userList = self.getUserFPData(fpData.username)
        if len(userList) >= 3:
            return _('alread has three fingerprint')
        
        userList.append(fpData)

        self.margeConfig(userList)
        
        return 'success'

    def updateFPData(self, fpData):
        if fpData is None or not isinstance(fpData, FPData):
            return _('not a fingerprint data!')
        
        data = self.findConfig(fpData)

        if not data:
            return _('fingerpring data not found!')
            
        # self.fpData.remove(data)
        # self.fpData.append(fpData)

        data.alias = fpData.alias
        
        return 'success'

    def rename(self, fpId, alias):
        username = os.getlogin()
        fpData = FPData(username, fpId, alias)
        ret = self.updateFPData(fpData)
        if ret == 'success':
            return self.commitConfig()
        else:
            return ''
    
    def deleteConfig(self, fpData):
        if not fpData:
            return _('empty fingerpring data')

        # self.fpData = filter(lambda data: fpData.username != fpData.username and fpData.fpId != data.fpId, self.fpData)
        for data in self.fpData:
            if data.username == fpData.username and data.fpId == fpData.fpId:
                self.fpData.remove(data)
                return 'success'
        return _('no fingerprint data!')

    def delete_id(self, fpId):
        username = os.getlogin()
        print username
        fpData = FPData(username, fpId, '')

        ret = self.deleteConfig(fpData)
        if ret == 'success':
            return self.commitConfig()
        else:
            return ''


    def findConfig(self, fpData):
        if not fpData:
            return None

        for data in self.fpData:
            if data.username == fpData.username and data.fpId == fpData.fpId:
                return data

        return None
    
    def findConfigUseFPId(self, fpId):
        if not fpId:
            return None

        for data in self.fpData:
            if data.fpId == fpId:
                return data

        return None

class Fingerprint(GObject.GObject):
    '''
    '''
    FEATURE_REPEAT = 1
    FEATURE_UNKNOW = 2
    NO_EMPTY_TEMPLATE_ERROR = 3
    DEVICE_INIT_ERROR = 4
    GET_IMAGE_INFO_ERROR = 5
    TEMPLATE_SAVE_ERROR = 6
    IO_ERROR = 7
    NO_DEVICE_ERROR = 8

    def __init__(self):
        print '__init__ FP'
        self.fp_dev = FPConfig()
        bus_loop = dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        self.mainloop = bus_loop
        self.system_bus = dbus.SystemBus(mainloop=bus_loop)
        self.fpObject = self.system_bus.get_object('com.nfs.Fingerprint', '/com/nfs/FingerprintObject')
        self.iface = dbus.Interface(self.fpObject, dbus_interface='com.nfs.fingerprint')
        self.enrolledFingerprint = self.iface.get_dbus_method("enrolledFingerprint", dbus_interface=None)
        self.enroll_end = self.iface.get_dbus_method("enrolledFingerprintCancel", dbus_interface=None)
        self.commitFpData = self.iface.get_dbus_method("commitFpData", dbus_interface=None)
        self.deleteTemplate = self.iface.get_dbus_method("deleteTemplate", dbus_interface=None)
        self.iface.connect_to_signal('capturingChange', self.signalHandler)

        GObject.GObject.__init__(self)


        GObject.signal_new('enroll-one-image', self, GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_INT, GObject.TYPE_STRING) )
        GObject.signal_new('enroll-finished', self, GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_BOOLEAN, ) )
        GObject.signal_new('info-changed', self, GObject.SignalFlags.RUN_LAST, None, () )
        GObject.signal_new('catched-canceled', self, GObject.SignalFlags.RUN_LAST, None, () )
        GObject.signal_new('catched-error', self, GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_PYOBJECT, ) )

        self.errorCode = {}
        self.errorCode['FEATURE_REPEAT'] = Fingerprint.FEATURE_REPEAT
        self.errorCode['FEATURE_UNKNOW'] = Fingerprint.FEATURE_UNKNOW
        self.errorCode['NO_EMPTY_TEMPLATE_ERROR'] = Fingerprint.NO_EMPTY_TEMPLATE_ERROR
        self.errorCode['DEVICE_INIT_ERROR'] = Fingerprint.DEVICE_INIT_ERROR
        self.errorCode['TEMPLATE_SAVE_ERROR'] = Fingerprint.TEMPLATE_SAVE_ERROR
        self.errorCode['IO_ERROR'] = Fingerprint.IO_ERROR
        self.errorCode['NO_DEVICE_ERROR'] = Fingerprint.NO_DEVICE_ERROR
        pass

    def enroll_start(self):
        self.enrolledFingerprint(os.getlogin())

    def signalHandler(self, msg):
        print 'handle %s ' %  msg
        username = os.getlogin()

        regex = r'' + username + ':([\s\S]*)'
        pattern = re.compile(regex)
        m = pattern.match(msg)
        if m:
            g = m.groups()
            if not g[0]:
                return
            else:
                msg = g[0]

        if 'capturing-exist' == msg:
            error = GObject.GError()
            error.code = Fingerprint.FEATURE_REPEAT
            self.emit('catched-error', error)
        
        if 'capturing-canceled' == msg:
            self.emit('catched-canceled')
        
        msg = msg.split(':') #capturing-tmp-count
        if isinstance(msg, list) and len(msg) > 1 and msg[0] == 'capturing-error':
            error = GObject.GError()
            if msg[1] in self.errorCode:
                error.code = self.errorCode[msg[1]]
            else:
                error.code = self.errorCode['FEATURE_UNKNOW']
            self.emit('catched-error', error)

        elif isinstance(msg, list) and len(msg) > 1 and msg[0] == 'capturing-tmp-count':
            count = int(msg[1])
            print 'emit enroll-one-image'
            self.emit('enroll-one-image', count, '')

        elif isinstance(msg, list) and len(msg) > 1 and msg[0] == 'capturing-id':
            fpId = int(msg[1])
            fpData = FPData(os.getlogin(), fpId, str(_('Fingerprint')) + str(msg[1]))
            ret = self.fp_dev.insertFPData(fpData)
            if ret == 'success':
                fpDataStr = self.fp_dev.commitConfig()
                ret = self.commitFpData(fpDataStr)
            if ret == 0:
                self.emit('enroll-finished', True)
                self.emit('info-changed')
            else:
                error = GObject.GError()
                error.code = Fingerprint.IO_ERROR
                print ret
                self.emit('catched-error', error)
        pass

    # def connect(self, handler):
    #     self.iface.connect_to_signal('verifyStatusChange', handler)

    def get_current_user_info_list(self):
        username = os.getlogin()
        return self.fp_dev.getUserFPData(username)

    def rename(self, fpId, alias):
        fpDataStr = self.fp_dev.rename(fpId, alias)
        ret = self.commitFpData(fpDataStr)
        if ret == 0:
            self.emit('info-changed')
        return ret

    def delete_id(self, fpId):
        ret = self.deleteTemplate(int(fpId))
        if ret != 0:
            return Fingerprint.IO_ERROR
        fpDataStr = self.fp_dev.delete_id(fpId)
        ret = self.commitFpData(fpDataStr)
        if ret == 0:
            self.emit('info-changed')
        return ret

    def quit(self):
        self.mainloop.quit()

if __name__ == "__main__":
    pass
